# encoding: utf-8

module Flappy
  module IOSLogger

    attr_accessor :logger_iOS
    attr_accessor :logger_iOS_build
    attr_accessor :logger_iOS_archive
    attr_accessor :logger_iOS_pod

    def initialize_iOS_logger(options)
      @options = options

      #### common
      log_file = File.open(File.join(@app_output_archive_log_path, "#{@start_time}_log.txt"), 'a')
      if options[:quiet] || options[:'no-savelog']
        log_file = '/dev/null' if options[:'no-savelog']
        Flappy.logger_iOS  = Logger.new(log_file)
      else
        Flappy.logger_iOS = Logger.new Flappy::MultiIO::MultiDelegator.delegate(:write, :close).to(STDOUT, log_file)
      end
      Flappy.logger_iOS.level = options[:verbose] ? Logger::INFO : Logger::ERROR


      ##### build
      log_file = File.open(File.join(@app_output_archive_log_path, "#{@start_time}_log_build.txt"), "a")
      if options[:quiet] || options[:'no-savelog']
        log_file = '/dev/null' if options[:'no-savelog']
        Flappy.logger_iOS_build  = Logger.new(log_file)
      else
        Flappy.logger_iOS_build = Logger.new Flappy::MultiIO::MultiDelegator.delegate(:write, :close).to(STDOUT, log_file)
      end
      Flappy.logger_iOS_build.level = options[:verbose] ? Logger::INFO : Logger::ERROR


      ##### archive
      log_file = File.open(File.join(@app_output_archive_log_path, "#{@start_time}_log_archive.txt"), "a")
      if options[:quiet] || options[:'no-savelog']
        log_file = '/dev/null' if options[:'no-savelog']
        Flappy.logger_iOS_archive  = Logger.new(log_file)
      else
        Flappy.logger_iOS_archive = Logger.new Flappy::MultiIO::MultiDelegator.delegate(:write, :close).to(STDOUT, log_file)
      end
      Flappy.logger_iOS_archive.level = options[:verbose] ? Logger::INFO : Logger::ERROR


      ##### pod
      log_file = File.open(File.join(@app_output_archive_log_path, "#{@start_time}_log_pod.txt"), "a")
      if options[:quiet] || options[:'no-savelog']
        log_file = '/dev/null' if options[:'no-savelog']
        Flappy.logger_iOS_pod  = Logger.new(log_file)
      else
        Flappy.logger_iOS_pod = Logger.new Flappy::MultiIO::MultiDelegator.delegate(:write, :close).to(STDOUT, log_file)
      end
      Flappy.logger_iOS_pod.level = options[:verbose] ? Logger::INFO : Logger::ERROR
    end


    def log_iOS(expression)
      if logger_iOS.nil?
        return
      end
      logger_iOS.info "#{expression}"
      puts "#{expression}"
    end

    def log_iOS_build(command)
      if logger_iOS_build.nil?
        return
      end
      if @options[:quiet] || @options[:'no-savelog']
        if @options[:quiet]
          logger_iOS_build.info `#{command}`
        else
          system(command)
        end
      else
        logger_iOS_build.info `#{command}`
      end
    end

    def log_iOS_archive(command)
      if logger_iOS_archive.nil?
        return
      end
      if @options[:quiet] || @options[:'no-savelog']
        if @options[:quiet]
          logger_iOS_archive.info `#{command}`
        else
          system(command)
        end
      else
        logger_iOS_archive.info `#{command}`
      end
    end

    def log_iOS_pod(command)
      if logger_iOS_pod.nil?
        return
      end
      if @options[:quiet] || @options[:'no-savelog']
        if @options[:quiet]
          logger_iOS_pod.info `#{command}`
        else
          system(command)
        end
      else
        logger_iOS_pod.info `#{command}`
      end
    end

  end
end
